// The nsIGCAChatIndexComp interface is used to record the chat indices for your contacts across browser instances

const GCACHATINDEXCOMP_CONTRACTID = '@mozilla.org/GChatAlertChtIndexComponent;1';
const GCACHATINDEXCOMP_CID = Components.ID('{0BAAB4A5-A489-11DA-957E-00E08161165F}');
const GCACHATINDEXCOMP_IID = Components.interfaces.nsIGCAChatIndexComp;

function nsGCAChatIndexComp()
{
    this.chatIndices = new Array();
    this.sentenceNumbers = new Array();
}

nsGCAChatIndexComp.prototype =
{
    // get the chat index for contact with email as "contactEmail"
    getChatIndex : function (contactEmail)
    {
        if (typeof(this.chatIndices[contactEmail]) == "undefined")
            return "";
        else
            return this.chatIndices[contactEmail];
    },
    
    // set the chat index for contact with email as "contactEmail"
    setChatIndex : function (contactEmail, chatIndex)
    {
        this.chatIndices[contactEmail] = chatIndex;
    },
    
    // get total sentence number in the lastest message of the specified contact
    getNumOfSentences : function (contactEmail)
    {
        if (typeof(this.sentenceNumbers[contactEmail]) == "undefined")
            return 0;
        else
            return this.sentenceNumbers[contactEmail];
    },
    
    // set total sentence number in the lastest message of the specified contact
    setNumOfSentences : function (contactEmail, numOfSentences)
    {
        this.sentenceNumbers[contactEmail] = numOfSentences;
    },
    
    
    QueryInterface: function (iid)
    {
        if (!iid.equals(Components.interfaces.nsISupports) && !iid.equals(GCACHATINDEXCOMP_IID))
            throw Components.results.NS_ERROR_NO_INTERFACE;

        return this;
    }
};

// the global chat idex service object
var gcaChatIndexService = null;

var nsGCAChatIndexCompModule =
{
    registerSelf: function (compMgr, fileSpec, location, type)
    {
        compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
        compMgr.registerFactoryLocation(GCACHATINDEXCOMP_CID,
                                        "GMail Chat Alert Component",
                                        GCACHATINDEXCOMP_CONTRACTID,
                                        fileSpec,
                                        location,
                                        type);
    },
    
    getClassObject: function (compMgr, cid, iid)
    {
        if (!cid.equals(GCACHATINDEXCOMP_CID))
            throw Components.results.NS_ERROR_NO_INTERFACE;
        if (!iid.equals(Components.interfaces.nsIFactory))
            throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
            
        return nsGCAChatIndexCompFactory;   
    },
    
    canUnload: function(compMgr)
    {
        return true;
    },
};

var nsGCAChatIndexCompFactory =
{
    // only one instance should be created
    createInstance: function (outer, iid)
    {
        if (outer != null)
            throw Components.results.NS_ERROR_NO_AGGREGATION;
        if (!iid.equals(GCACHATINDEXCOMP_IID) && !iid.equals(Components.interfaces.nsISupports))
            throw Components.results.NS_ERROR_INVALID_ARG;

        if (gcaChatIndexService == null)
            gcaChatIndexService = new nsGCAChatIndexComp();
        return gcaChatIndexService;
    }
};

function NSGetModule(comMgr, fileSpec)
{
    return nsGCAChatIndexCompModule;
}