const GChatAlertPrefsListener =
{
    observe : function (subject, topic, prefName)
    {
        if (topic != "nsPref:changed")
            return;
        
        switch (prefName)
        {
            case prefStringGChatAlertAudioFilePath:
                GChatAlert.onAlertAudioFilePathChanged();
                break;
            case prefStringGChatAlertUseDefaultAlertAudio:
                GChatAlert.onUseDefaultAlertAudioChanged();
                break;
            default:
                break;
        }
    },
};

var GChatAlert =
{
/*** 
    Const Definition
 ***/   
    gmailURI : "mail.google.com",                               // GMail URI
    chatPopupClass : "wackbody",                                // class name for the "TD" element in each chat popup
    chatDirAttr : "chatdir",                                    // attribute name representing chat direction
    chatIndexAttr : "chatindex",                                // attribute name representing index for each message
    incomingChatDir : "1",                                      // "chatdir" attribute value for incoming messages
    outgoingChatDir : "2",                                      //                           for outgoing messages
    checkInterval : 1500,                                       // checking interval in miliseconds
    chatIndexUndefined : "undefined",                           // chat index for messages from Google Talk
/***    
    End of Const Definition
 ***/

    // The following three instances will be used in each detection, so they will exist with GChatAlert object
    nsIChatIndices : null,                                      // nsIGCAChatIndexComp instance to store the chat indices for contacts
    nsISoundInstance : null,                                    // nsISound instance to play audible alert
    audioFileURI : null,                                        // nsIURI instance holding the current audio file for alerting
    
    init : function ()
    {
        // get nsIGCAChatIndexComp instance
        this.nsIChatIndices = Components.classes['@mozilla.org/GChatAlertChtIndexComponent;1']
                                        .createInstance(Components.interfaces.nsIGCAChatIndexComp);

        // get nsISound instance and initialize it
        this.nsISoundInstance = Components.classes["@mozilla.org/sound;1"]
                                          .createInstance(Components.interfaces.nsISound);
        this.nsISoundInstance.init();   // call the init() method to speed up the first alert
        
        this.getAlertAuido();
        
        // register the Pref Change listener
        var prefBranchInternal = gBrowser.mPrefs.QueryInterface(Components.interfaces.nsIPrefBranchInternal);
        prefBranchInternal.addObserver(prefStringGChatAlertUseDefaultAlertAudio, GChatAlertPrefsListener, false);
        prefBranchInternal.addObserver(prefStringGChatAlertAudioFilePath, GChatAlertPrefsListener, false);

        setTimeout(function () { GChatAlert.checkForNewMsg(); }, this.checkInterval);
    },
    
    deinit : function ()
    {
        var prefBranchInternal = gBrowser.mPrefs.QueryInterface(Components.interfaces.nsIPrefBranchInternal);
        prefBranchInternal.removeObserver(prefStringGChatAlertAudioFilePath, GChatAlertPrefsListener);
        prefBranchInternal.removeObserver(prefStringGChatAlertUseDefaultAlertAudio, GChatAlertPrefsListener);
    },
 
    getAlertAuido : function ()
    {
        var ioService = Components.classes["@mozilla.org/network/io-service;1"]
                                  .getService(Components.interfaces.nsIIOService);

        // Get the current alert audio setting
        if (gBrowser.mPrefs.getBoolPref(prefStringGChatAlertUseDefaultAlertAudio))   // use default alert audio
            this.audioFileURI = ioService.newURI(gchatAlertDefaultAlertAudioURI, null, null);
        else                                                                        // use local audio file
            this.audioFileURI = ioService.newFileURI(gBrowser.mPrefs.getComplexValue(prefStringGChatAlertAudioFilePath,
                                                                                     Components.interfaces.nsILocalFile));
    },
    
    // Get all the documents content in "aWindow"
    getAllDocuments : function (aWindow)
    {
        var allDocuments = new Array();
        
        if (aWindow)
        {
            var frames = aWindow.frames;
            var frameNums = frames.length;
            
            // If the document of the current window exists, then add it first
            if (aWindow.document)
                allDocuments.push(aWindow.document);
                
            // Loop through all the subframes and get documents contained
            for (var i = 0; i < frameNums; i++)
                allDocuments = allDocuments.concat(this.getAllDocuments(frames[i]));
        }
        
        return allDocuments;
    },
    
    // Get all the descendent element of the root element with the given class name
    getElementsByClassName : function (rootElement, strTagName, strClassName)
    {
        var arrElements = rootElement.getElementsByTagName(strTagName);
        var arrReturnElements = new Array();
        
        strClassName = strClassName.replace(/\-/g, "\\-");
        var oRegExp = new RegExp("(^|\\s)" + strClassName + "(\\s|$)");
        for (var i = 0; i < arrElements.length; i++)
            if (oRegExp.test(arrElements[i].className))
                arrReturnElements.push(arrElements[i]);
        
        return arrReturnElements;
    },

    // Get all the chat windows
    getAllChatPopups : function ()
    {
        var chatPopups = new Array();

        for (var i = gBrowser.tabContainer.childNodes.length - 1; i >= 0; i--)
        {
            var aBrowser = gBrowser.getBrowserAtIndex(i);
            if (aBrowser.currentURI.spec.toLowerCase().indexOf(this.gmailURI) >= 0)
            {
                // Get all the documents contained in GMail page
                var allGMailDocuments = this.getAllDocuments(aBrowser.contentWindow);
                // Find the chat popup in current page
                var gmailDocNums = allGMailDocuments.length;
                for (var j = 0; j < gmailDocNums; j++)
                    chatPopups = chatPopups.concat(this.getElementsByClassName(allGMailDocuments[j],
                                                                               "TD",                    // tag name
                                                                               this.chatPopupClass));   // class name
            }
        }
        
        // no gmail tab
        return chatPopups;
    },
    
    newMsgArrived : function ()
    {
        this.nsISoundInstance.play(this.audioFileURI);
    },
    
    checkForNewMsg : function ()
    {
        // Trigger the checking again (the code is brought here because putting it at the end of the
        // function will stop the checking when something goes wrong in the code for detection
        setTimeout(function () { GChatAlert.checkForNewMsg(); }, this.checkInterval);
        
        var allChatPopups = this.getAllChatPopups();
        var numChatPopups = allChatPopups.length;

        /*** The DOM structure:
            TD .wackbody
            |
            |---DIV #bc_historyN
                |
                |---BR
                |
                |---SPAN (this node contains messages sent from one speaker continuously)
                |   |           attrbute chatdir: "1" for incoming messages, "2" for outgoing
                |   |
                |   |---DIV .bz_msg (this node contains several DIV nodes, one for each sentence)
                |       |
                |       |---DIV
                |       |   |
                |       |   |---B (name of the speaker)
                |       |   |---SPAN (the words)    (these two nodes is the first sentence from one speaker)
                |       |               this span node has the "chatindex" attribute
                |       |
                |       |---DIV .bz_msg_cont (zero or mode such nodes for others sentence from the same speaker
                |       |               this node also has the "chatindex" attribute
                |
                |
                |---SPAN (attrbute chatdir: "")
                |   |
                |   |---DIV .bz_msg (this node contains several DIV nodes, one for each sentence)
                |       |
                |       |---DIV
                |           |
                |           |---SPAN .bz_history_info ("sent at ...")
                |
                |   
                |---SPAN (attrbute chatdir: "")
                |   |
                |   |---DIV .bz_msg (this node contains several DIV nodes, one for each sentence)
                |   |   |
                |       |---DIV
                |           |---SPAN .bz_history_busy ("Sam is busy.  You may be interrupting.")
                |
                |
                |---SPAN
                |
                |---...
                |
                |___DIV .bz_blankline
            
         ***/
        /***
              Bug found on 2006/02/23.
              If a chat popup is closed with incoming message as the last message,
              sometimes there will be no alert if the next message from the same contact has the exactly number
              of sentences with the one before the popup is closed.
              So using the "chatindex" attribute provided is a much better solution
         ***/
        /***
              Since messages from Google Talk will have no chat index, we do the detection of new messages in two
              ways: by chat index (for messages from GMail, for accuracy) or by sentence numbers
         ***/
        for (var i = 0; i < numChatPopups; i++)
        {
            // the contact email (with "wkbdy_" as prefix), as an index
            var contactEmail = allChatPopups[i].id;
            
            // get the last granchild node ("SPAN") of the "TD" element which has "chatdir" attribute as "1" or "2",
            // i.e., is a node representing either incoming or outgoing messages
            var grandChildNodes = allChatPopups[i].firstChild.childNodes;
            var grandChildNodeNums = grandChildNodes.length;            
            var latestMsgNode = null;
            var chatDir = "";
            
            for (var j = grandChildNodeNums - 1; j >= 0; j--)
            {
                if (grandChildNodes[j].hasAttribute(this.chatDirAttr))
                {
                    chatDir = grandChildNodes[j].getAttribute(this.chatDirAttr);
                    if ((chatDir == this.incomingChatDir) || (chatDir == this.outgoingChatDir))
                    {
                        latestMsgNode = grandChildNodes[j];
                        break;
                    }
                }
            }
            
            // Only when there are message nodes and the latest message is incoming message, need we do the checking
            if (latestMsgNode != null)
            {
                if (chatDir == this.incomingChatDir)
                {
                    var newMsgArrived = false;
                    
                    // get the chat index for this contact
                    var lastSentenceNode = latestMsgNode.firstChild.lastChild;
                    var latestSentenceNum = latestMsgNode.firstChild.childNodes.length;
                    var chatIndex = "";
                    
                    if (lastSentenceNode.hasAttribute(this.chatIndexAttr))                  // more that one sentence in the latest message
                        chatIndex = lastSentenceNode.getAttribute(this.chatIndexAttr);
                    else if (lastSentenceNode.lastChild.hasAttribute(this.chatIndexAttr))   // only one sentence
                        chatIndex = lastSentenceNode.lastChild.getAttribute(this.chatIndexAttr);
                
                    if (chatIndex != this.chatIndexUndefined)   // messages with chat index defined (from GMail Chat)
                    {
                        if (chatIndex != this.nsIChatIndices.getChatIndex(contactEmail))
                        {
                            newMsgArrived = true;
                            this.nsIChatIndices.setChatIndex(contactEmail, chatIndex);
                            // clear the sentence number for this contact, in case one just switch talk using
                            // Google talk and GMail
                            this.nsIChatIndices.setNumOfSentences(contactEmail, 0);
                        }
                    }
                    else                                        // messages without chat index (from Google Talk)
                    {                                           // check the sentence number in latest message
                        if (latestSentenceNum != this.nsIChatIndices.getNumOfSentences(contactEmail))
                        {
                            newMsgArrived = true;
                            this.nsIChatIndices.setNumOfSentences(contactEmail, latestSentenceNum);
                        }
                    }
                    
                    if (newMsgArrived)
                        setTimeout(function () {GChatAlert.newMsgArrived(); }, 0);
                }
                else        // if "latestMsgNode" is not null, the message node will be either "incoming" or "outgoing"
                    this.nsIChatIndices.setNumOfSentences(contactEmail, 0);
            }
        }
    },
    
    // preference change event handler
    onUseDefaultAlertAudioChanged : function ()
    {
        this.getAlertAuido();
    },
    
    onAlertAudioFilePathChanged : function ()
    {
        this.getAlertAuido();
    },
};

window.addEventListener("load", function () { GChatAlert.init(); }, false);
window.addEventListener("unload", function () { GChatAlert.deinit(); }, false);
